import json

from automon import Logging
from automon.integrations.requests import Requests

from .config import PhantomConfig
from .artifact import Artifact
from .container import Container
from .rest import Urls

log = Logging(name='PhantomClient', level=Logging.DEBUG)


class PhantomClient:
    def __init__(self, host: str = None,
                 user: str = None,
                 password: str = None,
                 config: PhantomConfig = None):
        """Phantom Client"""

        self.config = config or PhantomConfig(host=host, user=user, password=password)
        self.client = Requests(headers=self.config.headers)

        self.action_run = None
        self.app = None
        self.app_run = None
        self.artifacts = None
        self.asset = None
        self.cluster_node = None
        self.containers = None
        self.playbook_run = None

    def __repr__(self) -> str:
        return f'{self.__dict__}'

    def _content(self) -> bytes:
        """get result"""
        return self.client.results.content

    def _content_dict(self) -> dict:
        """convert request.content to dict"""
        return json.loads(self._content())

    def _get(self, url: str) -> bool:
        """send get request"""
        return self.client.get(url=url, headers=self.client.headers)

    def _delete(self, url: str) -> bool:
        """send get request"""
        return self.client.delete(url=url, headers=self.client.headers)

    def _post(self, url: str, data: dict) -> bool:
        """send post request"""
        return self.client.post(url=url, headers=self.client.headers, data=data)

    def create_artifact(
            self,
            container_id,
            cef=None,
            cef_types=None,
            data=None,
            description=None,
            end_time=None,
            ingest_app_id=None,
            kill_chain=None,
            label=None,
            name=None,
            owner_id=None,
            run_automation=None,
            severity=None,
            source_data_identifier=None,
            start_time=None,
            tags=None,
            type=None,
            *args, **kwargs) -> Artifact:
        """Create artifact"""

        artifact = Artifact(
            dict(cef=cef,
                 cef_types=cef_types,
                 container_id=container_id,
                 data=data,
                 description=description,
                 end_time=end_time,
                 ingest_app_id=ingest_app_id,
                 kill_chain=kill_chain,
                 label=label,
                 name=name,
                 owner_id=owner_id,
                 run_automation=run_automation,
                 severity=severity,
                 source_data_identifier=source_data_identifier,
                 start_time=start_time,
                 tags=tags,
                 type=type)
        )

        if self._post(Urls().artifact(*args, **kwargs), data=artifact.to_json()):
            if self.client.results.status_code == 200:
                id = self.client.to_dict()['id']
                log.info(f'artifact created. {artifact} {self.client.to_dict()}')
                return self.list_artifact(artifact_id=id)
            else:
                existing_artifact_id = self.client.to_dict()['existing_artifact_id']
                log.info(f'artifact exists. {artifact} {self.client.to_dict()}')
                return self.list_artifact(artifact_id=existing_artifact_id)

        log.error(f'create artifact. {self.client.to_dict()}', enable_traceback=False)
        return False

    def create_container(
            self,
            label,
            name,
            artifacts=None,
            asset_id=None,
            close_time=None,
            custom_fields=None,
            data=None,
            description=None,
            due_time=None,
            end_time=None,
            ingest_app_id=None,
            kill_chain=None,
            owner_id=None,
            role_id=None,
            run_automation=None,
            sensitivity=None,
            severity=None,
            source_data_identifier=None,
            start_time=None,
            open_time=None,
            status=None,
            tags=None,
            tenant_id=None,
            container_type=None,
            template_id=None,
            authorized_users=None,
            *args, **kwargs) -> Container:
        """Create container"""

        container = Container(
            dict(label=label,
                 name=name,
                 artifacts=artifacts,
                 asset_id=asset_id,
                 close_time=close_time,
                 custom_fields=custom_fields,
                 data=data,
                 description=description,
                 due_time=due_time,
                 end_time=end_time,
                 ingest_app_id=ingest_app_id,
                 kill_chain=kill_chain,
                 owner_id=owner_id,
                 role_id=role_id,
                 run_automation=run_automation,
                 sensitivity=sensitivity,
                 severity=severity,
                 source_data_identifier=source_data_identifier,
                 start_time=start_time,
                 open_time=open_time,
                 status=status,
                 tags=tags,
                 tenant_id=tenant_id,
                 container_type=container_type,
                 template_id=template_id,
                 authorized_users=authorized_users)
        )

        if self._post(Urls().container(*args, **kwargs), data=container.to_json()):
            if self.client.results.status_code == 200:
                id = self.client.to_dict()['id']
                log.info(f'container created. {container} {self.client.to_dict()}')
                return self.list_containers(container_id=id)
        log.error(f'create container. {self.client.to_dict()}', enable_traceback=False)
        return False

    def delete_container(self, container_id, *args, **kwargs):
        """Delete containers"""
        assert isinstance(container_id, int)

        if self._delete(Urls().container(identifier=container_id, *args, **kwargs)):
            if self.client.results.status_code == 200:
                log.info(f'container deleted: {container_id}')
                return True
        log.error(f'delete container: {container_id}. {self.client.to_dict()}', enable_traceback=False)
        return False

    def isConnected(self) -> bool:
        """check if client can connect"""
        if self._get(Urls().container()):
            log.info(f'Phantom client check. '
                     f'[{self.client.results.status_code}] '
                     f'{self.config.host}')
            return True

        else:
            log.error(f'Phantom client failed.')
        return False

    def list_artifact(self, artifact_id: int = None, **kwargs) -> bool:
        """list action run"""
        if self._get(Urls().artifact(identifier=artifact_id, **kwargs)):
            request = self._content_dict()
            if artifact_id:
                return Artifact(request)
            artifacts = [Artifact(a) for a in request['data']]
            return artifacts
        return []

    def list_action_run(self, **kwargs) -> bool:
        """list action run"""
        if self._get(Urls().action_run(**kwargs)):
            self.containers = self._content_dict()
            return True
        return False

    def list_app(self, **kwargs) -> bool:
        """list app"""
        if self._get(Urls().app(**kwargs)):
            self.app = self._content_dict()
            return True
        return False

    def list_app_run(self, **kwargs) -> bool:
        """list app run"""
        if self._get(Urls().app_run(**kwargs)):
            self.app_run = self._content_dict()
            return True
        return False

    def list_artifacts(self, **kwargs) -> bool:
        """list artifacts"""
        if self._get(Urls().artifact(**kwargs)):
            self.artifacts = self._content_dict()
            return True
        return False

    def list_asset(self, **kwargs) -> bool:
        """list asset"""
        if self._get(Urls().asset(**kwargs)):
            self.asset = self._content_dict()
            return True
        return False

    def list_containers(self,
                        container_id=None,
                        page=None,
                        page_size=10,
                        *args, **kwargs) -> [Container]:
        """list containers"""
        if self._get(Urls().container(identifier=container_id, page=page, page_size=page_size, *args, **kwargs)):
            request = self._content_dict()
            if container_id:
                return Container(request)
            containers = [Container(c) for c in request['data']]
            return containers
        return []

    def list_cluster_node(self, **kwargs) -> bool:
        """list cluster node"""
        if self._get(Urls().cluster_node(**kwargs)):
            self.cluster_node = self._content_dict()
            return True
        return False

    def list_playbook_run(self, **kwargs) -> bool:
        """list cluster node"""
        if self._get(Urls().playbook_run(**kwargs)):
            self.playbook_run = self._content_dict()
            return True
        return False

    def list_vault(self, identifier=None, **kwargs) -> bool:
        """list cluster node"""
        if self._get(Urls().vault(identifier=identifier, **kwargs)):
            self.playbook_run = self._content_dict()
            return True
        return False



from automon import Logging
from automon.helpers.os import environ

log = Logging(name='PhantomConfig', level=Logging.DEBUG)


class PhantomConfig:
    def __init__(self, host: str = None,
                 user: str = None,
                 password: str = None,
                 auth_token: str = None):
        """Phantom Config"""

        self.host = host or environ('PHANTOM_HOST') or ''
        self.user = user or environ('PHANTOM_USER')
        self.password = password or environ('PHANTOM_PASSWORD')
        self.auth = (self.user, self.password)
        self.auth_token = auth_token or environ('PHANTOM_AUTH_TOKEN')

        self.headers = {'ph-auth-token': self.auth_token}

        if not self.host:
            log.warn(f'missing PHANTOM_HOST')

    def __repr__(self):
        return f'{self.__dict__}'



import io
import socket
import datetime
import functools

from minio import Minio

from automon.log import Logging
from automon.integrations.minio.config import MinioConfig

log = Logging(name='minio', level=Logging.INFO)


class MinioClient(object):

    def __init__(self, endpoint: str = None,
                 access_key: str = None,
                 secret_key: str = None,
                 config: MinioConfig = None):
        """Minio client
        """

        self._log = Logging(name=MinioClient.__name__, level=Logging.DEBUG)

        self.config = config or MinioConfig(endpoint=endpoint, access_key=access_key, secret_key=secret_key)

        self.client = self._client()
        self._lastClientCheck = None

    def __repr__(self):
        return f'endpoint: {self.config.endpoint} connected: {self.isConnected()}'

    def _client(self):
        client = Minio(
            endpoint=self.config.endpoint,
            access_key=self.config.access_key,
            secret_key=self.config.secret_key,
            session_token=self.config.session_token,
            secure=self.config.secure,
            region=self.config.region,
            http_client=self.config.http_client
        )
        return client

    def _sessionExpired(self):
        if not self._lastClientCheck:
            self._lastClientCheck = datetime.datetime.now()
            return True

        isExpired = self._lastClientCheck + datetime.timedelta(minutes=5)
        if self._lastClientCheck > isExpired:
            return True
        return False

    def isConnected(func):
        """Decorator that checks if MinioClient is connected
        """

        @functools.wraps(func)
        def _wrapper(self, *args, **kwargs):
            if not self._sessionExpired() or self.client.list_buckets():
                return func(self, *args, **kwargs)

        return _wrapper

    @isConnected
    def download_object(self, bucket, file):
        """ Minio object downloader
        """
        self._log.logging.debug(f'[downloader] Downloading: {bucket}/{file.object_name}')
        return self.client.get_object(bucket, file.object_name)

    @isConnected
    def list_objects(self, bucket: str, folder: str = None, recursive: bool = True):
        """ List Minio objects
        """
        self._log.logging.debug(f'[list_all_objects] bucket: {bucket}, folder: {folder}')
        return self.client.list_objects(bucket, folder, recursive=recursive)

    @isConnected
    def list_buckets(self, bucket: str = None) -> list:
        """ List Minio buckets
        """
        buckets = self.client.list_buckets()

        if bucket:
            self._log.debug(f'bucket name: {bucket}')
            return [x for x in buckets if x == bucket]

        self._log.logging.debug(f'buckets: {len(buckets)} {[x.name for x in buckets]}')
        return buckets

    @isConnected
    def put_object(self, bucket_name: str, object_name: str, data: io.BytesIO, length: int = None,
                   content_type='application/octet-stream',
                   metadata=None, sse=None, progress=None,
                   part_size=None):
        """ Minio object uploader
        """

        self._log.debug(f'[{self.put_object.__name__}] Uploading: {object_name}')

        length = length or data.getvalue().__len__()

        try:
            put = self.client.put_object(bucket_name=bucket_name, object_name=object_name,
                                         data=data, length=length,
                                         content_type=content_type,
                                         metadata=metadata, sse=sse, progress=progress)
            self._log.info(
                f'[put_object] Saved to '
                f'{self.client._endpoint_url}/{bucket_name}/{object_name}'
            )

            return put

        except Exception as _:
            self._log.error(
                f'[{self.put_object.__name__}] Unable to save '
                f'{self.client._endpoint_url}/{bucket_name}/{bucket_name}'
            )

            return False

    @isConnected
    def clear_bucket(self, bucket_name, folder=None):
        objects = self.list_objects(bucket_name, folder)
        for a in objects:
            print(a)
        return self.client.remove_objects(bucket_name, objects)

    @isConnected
    def make_bucket(self, bucket_name) -> bool:
        try:
            self._log.debug(f'Created bucket: {bucket_name}')
            self.client.make_bucket(bucket_name)
            return True

        except Exception as e:
            self._log.error(f'Bucket exists: {bucket_name} {e}', enable_traceback=False)
            return False

    @isConnected
    def remove_bucket(self, bucket_name) -> bool:
        try:
            self._log.debug(f'Removed bucket: {bucket_name}')
            self.client.remove_bucket(bucket_name)
            return True

        except Exception as e:
            self._log.error(f'Bucket does not exist: {bucket_name} {e}', enable_traceback=False)
            return False


def check_connection(host, port):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(1)
        s.connect((host, port))
        s.close()
        return True
    except Exception as e:
        log.error(e, enable_traceback=False)
        return False



import unittest

from automon.integrations.phantom import PhantomClient

c = PhantomClient()


class TestPhantomClient(unittest.TestCase):

    def test_isConnected(self):
        if c.isConnected():
            self.assertTrue(c.isConnected())
        else:
            self.assertFalse(c.isConnected())

    def test_create_artifact(self):
        if c.isConnected():
            id = c.create_container(label='testing', name='testing').id
            self.assertTrue(c.create_artifact(container_id=id))
        else:
            self.assertFalse(c.create_artifact(container_id=0))

    def test_create_container(self):

        if c.isConnected():
            self.assertTrue(c.create_container(label='testing', name='testing'))
        else:
            self.assertFalse(c.create_container(label='testing', name='testing'))

    def test_delete_containers(self):

        if c.isConnected():
            id = c.create_container(label='testing', name='testing').id
            self.assertTrue(c.delete_container(container_id=id))
        else:
            self.assertFalse(c.delete_container(container_id=0))

    def test_list_artifact(self):
        if c.isConnected():
            self.assertTrue(c.list_artifact())

    def test_list_containers(self):
        if c.isConnected():
            self.assertTrue(c.list_containers())
        else:
            self.assertFalse(c.list_containers())


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.phantom import PhantomConfig


class TestPhantomConfig(unittest.TestCase):
    c = PhantomConfig()

    def test_config(self):
        self.assertTrue(self.c)


if __name__ == '__main__':
    unittest.main()



import unittest
import hashlib

from automon.integrations.minio.client import MinioClient


class Client(unittest.TestCase):

    def test_MinioClient(self):
        self.assertTrue(MinioClient())
        self.assertTrue(MinioClient)

    def test_publicServer(self):
        MINIO_ENDPOINT = 'play.minio.io:9000'
        MINIO_ACCESS_KEY = 'Q3AM3UQ867SPQQA43P2F'
        MINIO_SECRET_KEY = 'zuf+tfteSlswRu7BJ86wekitnifILbZam1KYY3TG'

        m = MinioClient(endpoint=MINIO_ENDPOINT, access_key=MINIO_ACCESS_KEY, secret_key=MINIO_SECRET_KEY)
        bucket = hashlib.md5(f'{MINIO_ENDPOINT}'.encode()).hexdigest()

        if m.isConnected():

            self.assertTrue(len(m.list_buckets()) >= 0)   # this might fail if no buckets
            self.assertTrue(type(m.list_buckets()), list)
            self.assertTrue(type(m.list_buckets(bucket)), list)

            if m.list_buckets(bucket):
                self.assertTrue(m.remove_bucket(bucket))
            else:
                self.assertTrue(m.make_bucket(bucket))


if __name__ == '__main__':
    unittest.main()
